require 'spec_helper'

describe Mooset::Endpoints::Gitlab::UserSearch do
  let(:connection) { double }
  subject(:user_search) { Mooset::Endpoints::Gitlab::UserSearch.new(double(connection: connection, provider: 'ldap')) }

  let(:user1) { OpenStruct.new({
    name: "Admin",
    username: 'root',
    id: 1,
    state: 'active',
    avatar_url: '',
    email: 'root@gitlab',
    identities: [OpenStruct.new({extern_uid: 'ldap_id', provider: 'ldap'})]
  }) }

  let(:user2) { OpenStruct.new({
    name: "User2",
    username: 'user2',
    id: 1,
    state: 'active',
    avatar_url: '',
    email: 'user2@local',
    identities: [OpenStruct.new({extern_uid: 'ldap_user2', provider: 'ldap'})]
  }) }

  describe 'find' do
    describe '#all' do
      context 'without a block' do
        it 'returns an Enumerator' do
          expect(user_search.all).to be_a(Enumerator)
        end
      end

      it 'returns all the users' do
        expect(connection).to receive(:users).with(page: 1).and_return([user1])
        expect(connection).to receive(:users).with(page: 2).and_return([])

        @users = user_search.all.to_a
        expect(@users.length).to eq(1)

        expect(@users.first.id).to eq(1)
        expect(@users.first.gitlab_id).to eq(1)
        expect(@users.first.username).to eq('root')
        expect(@users.first.full_name).to eq('Admin')
        expect(@users.first.state).to eq('active')
        expect(@users.first.email).to eq('root@gitlab')
        expect(@users.first.access_level).to eq(nil)
        expect(@users.first.ldap_id).to eq('ldap_id')
      end

      describe ':id' do
        before do
          expect(connection).to receive(:user).with(1).and_return(user1)
          @user = user_search.find(id: 1)
        end

        it "returns the user resource" do
          expect(@user.id).to eq(1)
          expect(@user.gitlab_id).to eq(1)
          expect(@user.username).to eq('root')
          expect(@user.full_name).to eq('Admin')
          expect(@user.state).to eq('active')
          expect(@user.email).to eq('root@gitlab')
          expect(@user.access_level).to eq(nil)
          expect(@user.ldap_id).to eq('ldap_id')
        end
      end

      describe 'slow finders' do
        before do
          expect(connection).to receive(:users).with(page: 1).and_return([user1, user2])
          expect(connection).to receive(:users).with(page: 2).and_return([])
        end

        describe ':username' do
          it 'returns the correct user' do
            @user = user_search.find(username: 'user2')
            expect(@user.username).to eq('user2')
          end
        end

        describe ':email' do
          it 'returns the correct user' do
            @user = user_search.find(email: 'user2@local')
            expect(@user.email).to eq('user2@local')
          end
        end

        describe ':dn' do
          it 'returns the correct user' do
            @user = user_search.find(dn: 'ldap_user2')
            expect(@user.ldap_id).to eq('ldap_user2')
          end
        end
      end
    end
  end

  describe '#create' do
    it 'creates a gitlab user' do
      expect(connection).to receive(:create_user).with('email@local', 'password', {}).and_return(user1)
      @user = user_search.create('email@local', 'password')

      expect(@user.username).to eq('root')
    end
  end

  describe '#import' do
    describe 'from ldap' do
      before do
        @ldap_user = OpenStruct.new({
          full_name: 'Full name',
          username: 'username',
          dn: 'dn',
          provider: 'ldap',
          email: 'email@local'
        })
      end

      it 'creates a gitlab user' do
        expect(connection).to receive(:create_user).with('email@local', anything, {
          name: 'Full name',
          username: 'username',
          extern_uid: 'dn',
          provider: 'ldap',
          confirm: false
        }).and_return(user1)

        @user = user_search.import(@ldap_user)
      end
    end
  end

  describe '#group_members' do
    context 'without block' do
      it 'returns an Enumerator' do
        expect(user_search.group_members(1)).to be_a(Enumerator)
      end
    end
  end
end

